@@ -179,8 +179,10 @@ _cairo_image_surface_create_from_jpeg (GInputStream  *istream,
 	guint32                        pixel;
 	unsigned char                 *p_buffer;
 	int                            x;
+	gboolean                       read_all_scanlines = FALSE;
 
 	image = gth_image_new ();
+	surface = NULL;
 
 	if (! _g_input_stream_read_all (istream,
 			      	        &in_buffer,
@@ -228,11 +230,8 @@ _cairo_image_surface_create_from_jpeg (GInputStream  *istream,
 
 	jpeg_create_decompress (&srcinfo);
 
-	if (sigsetjmp (jsrcerr.setjmp_buffer, 1)) {
-		g_free (in_buffer);
-		jpeg_destroy_decompress (&srcinfo);
-		return image;
-	}
+	if (sigsetjmp (jsrcerr.setjmp_buffer, 1))
+		goto stop_loading;
 
 	_jpeg_memory_src (&srcinfo, in_buffer, in_buffer_size);
 
@@ -539,52 +538,58 @@ _cairo_image_surface_create_from_jpeg (GInputStream  *istream,
 		break;
 	}
 
+	read_all_scanlines = TRUE;
+
 	stop_loading:
 
-	cairo_surface_mark_dirty (surface);
+	if (surface != NULL) {
+		cairo_surface_mark_dirty (surface);
 
-	if (! g_cancellable_is_cancelled (cancellable)) {
-		int original_width;
-		int original_height;
+		if (! g_cancellable_is_cancelled (cancellable)) {
+			int original_width;
+			int original_height;
 
-		/* Set the original dimensions */
+			/* Set the original dimensions */
 
-		if ((orientation == GTH_TRANSFORM_ROTATE_90)
-		     ||	(orientation == GTH_TRANSFORM_ROTATE_270)
-		     ||	(orientation == GTH_TRANSFORM_TRANSPOSE)
-		     ||	(orientation == GTH_TRANSFORM_TRANSVERSE))
-		{
-			original_width = srcinfo.image_height;
-			original_height = srcinfo.image_width;
-		}
-		else {
-			original_width = srcinfo.image_width;
-			original_height = srcinfo.image_height;
+			if ((orientation == GTH_TRANSFORM_ROTATE_90)
+			     ||	(orientation == GTH_TRANSFORM_ROTATE_270)
+			     ||	(orientation == GTH_TRANSFORM_TRANSPOSE)
+			     ||	(orientation == GTH_TRANSFORM_TRANSVERSE))
+			{
+				original_width = srcinfo.image_height;
+				original_height = srcinfo.image_width;
+			}
+			else {
+				original_width = srcinfo.image_width;
+				original_height = srcinfo.image_height;
+			}
+
+			_cairo_metadata_set_original_size (metadata, original_width, original_height);
+
+			if (original_width_p != NULL)
+				*original_width_p = original_width;
+			if (original_height_p != NULL)
+				*original_height_p = original_height;
+			if (loaded_original_p != NULL)
+				*loaded_original_p = ! load_scaled;
+
+			/*_cairo_image_surface_set_attribute_int (surface, "Image::Rotation", rotation); FIXME*/
+			/* FIXME _cairo_image_surface_set_attribute (surface, "Jpeg::ColorSpace", jpeg_color_space_name (srcinfo.jpeg_color_space)); */
+
+			gth_image_set_cairo_surface (image, surface);
 		}
+		else
+			g_set_error_literal (error, G_IO_ERROR, G_IO_ERROR_CANCELLED, "");
 
-		_cairo_metadata_set_original_size (metadata, original_width, original_height);
-
-		if (original_width_p != NULL)
-			*original_width_p = original_width;
-		if (original_height_p != NULL)
-			*original_height_p = original_height;
-		if (loaded_original_p != NULL)
-			*loaded_original_p = ! load_scaled;
+		cairo_surface_destroy (surface);
+		surface = NULL; /* ignore other jpeg errors */
+	}
 
+	if (read_all_scanlines)
 		jpeg_finish_decompress (&srcinfo);
-		jpeg_destroy_decompress (&srcinfo);
-
-		/*_cairo_image_surface_set_attribute_int (surface, "Image::Rotation", rotation); FIXME*/
-		/* FIXME _cairo_image_surface_set_attribute (surface, "Jpeg::ColorSpace", jpeg_color_space_name (srcinfo.jpeg_color_space)); */
-
-		gth_image_set_cairo_surface (image, surface);
-	}
-	else {
-		jpeg_destroy_decompress (&srcinfo);
-		g_set_error_literal (error, G_IO_ERROR, G_IO_ERROR_CANCELLED, "");
-	}
-
-	cairo_surface_destroy (surface);
+	else
+		jpeg_abort_decompress (&srcinfo);
+	jpeg_destroy_decompress (&srcinfo);
 	g_free (in_buffer);
 
 	return image;
