@@ -45,13 +45,6 @@
 static void gth_viewer_page_interface_init (GthViewerPageInterface *iface);
 
 
-G_DEFINE_TYPE_WITH_CODE (GthImageViewerPage,
-			 gth_image_viewer_page,
-			 G_TYPE_OBJECT,
-			 G_IMPLEMENT_INTERFACE (GTH_TYPE_VIEWER_PAGE,
-					        gth_viewer_page_interface_init))
-
-
 static const GActionEntry actions[] = {
 	{ "image-zoom-in", gth_browser_activate_image_zoom_in },
 	{ "image-zoom-out", gth_browser_activate_image_zoom_out },
@@ -116,6 +109,14 @@ struct _GthImageViewerPagePrivate {
 };
 
 
+G_DEFINE_TYPE_WITH_CODE (GthImageViewerPage,
+			 gth_image_viewer_page,
+			 G_TYPE_OBJECT,
+			 G_ADD_PRIVATE (GthImageViewerPage)
+			 G_IMPLEMENT_INTERFACE (GTH_TYPE_VIEWER_PAGE,
+						gth_viewer_page_interface_init))
+
+
 static void
 gth_image_viewer_page_file_loaded (GthImageViewerPage *self,
 				   gboolean            success)
@@ -166,7 +167,7 @@ _gth_image_preloader_get_requested_size_for_next_images (GthImageViewerPage *sel
 		requested_size = -1;
 		break;
 	default:
-		requested_size = get_viewer_size (self);
+		requested_size = (int) floor ((double) get_viewer_size (self) * 0.5);
 		break;
 	}
 
@@ -260,7 +261,7 @@ profile_ready_cb (GObject      *source_object,
 	ProfileData        *profile_data = user_data;
 	GthImageViewerPage *self = profile_data->self;
 
-	if (self->priv->active) {
+	if (self->priv->active && ! self->priv->image_changed && _g_file_equal (self->priv->file_data->file, profile_data->file_data->file)) {
 		GthICCProfile *profile;
 
 		profile = gth_color_manager_get_profile_finish (GTH_COLOR_MANAGER (source_object), res, NULL);
@@ -428,16 +429,43 @@ _g_mime_type_can_load_different_quality (const char *mime_type)
 }
 
 
+typedef struct {
+	GthImageViewerPage *self;
+	GthFileData *file_data;
+} UpdateQualityData;
+
+
+static void
+update_quality_data_free (UpdateQualityData *data)
+{
+	_g_object_unref (data->file_data);
+	g_free (data);
+}
+
+
 static gboolean
 update_quality_cb (gpointer user_data)
 {
-	GthImageViewerPage *self = user_data;
+	UpdateQualityData  *data = user_data;
+	GthImageViewerPage *self = data->self;
+	gboolean            file_changed;
+
+	if (! _gth_image_viewer_page_load_with_preloader_finish (self)) {
+		update_quality_data_free (data);
+		return FALSE;
+	}
 
 	if (self->priv->update_quality_id != 0) {
 		g_source_remove (self->priv->update_quality_id);
 		self->priv->update_quality_id = 0;
 	}
 
+	file_changed = ! _g_file_equal_uris (data->file_data->file, self->priv->file_data->file);
+	update_quality_data_free (data);
+
+	if (file_changed)
+		return FALSE;
+
 	if (! self->priv->active)
 		return FALSE;
 
@@ -479,9 +507,16 @@ update_image_quality_if_required (GthImageViewerPage *self)
 		self->priv->update_quality_id = 0;
 	}
 
+	UpdateQualityData *data;
+
+	data = g_new0 (UpdateQualityData, 1);
+	data->self = self;
+	data->file_data = _g_object_ref (self->priv->file_data);
+
+	_g_object_ref (self);
 	self->priv->update_quality_id = g_timeout_add (UPDATE_QUALITY_DELAY,
 						       update_quality_cb,
-						       self);
+						       data);
 #endif
 }
 
@@ -838,7 +873,7 @@ overview_leave_notify_event_cb (GtkWidget *widget,
 	GthImageViewerPage *self = data;
 
 	if (widget == self->priv->overview)
-		self->priv->pointer_on_overview = FALSE;
+		self->priv->pointer_on_overview = gth_image_overview_get_scrolling_is_active (GTH_IMAGE_OVERVIEW (self->priv->overview));
 
 	return FALSE;
 }
@@ -1899,8 +1934,6 @@ gth_image_viewer_page_finalize (GObject *obj)
 static void
 gth_image_viewer_page_class_init (GthImageViewerPageClass *klass)
 {
-	g_type_class_add_private (klass, sizeof (GthImageViewerPagePrivate));
-
 	G_OBJECT_CLASS (klass)->finalize = gth_image_viewer_page_finalize;
 }
 
@@ -1932,7 +1965,7 @@ gth_image_viewer_page_init (GthImageViewerPage *self)
 {
 	int i;
 
-	self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, GTH_TYPE_IMAGE_VIEWER_PAGE, GthImageViewerPagePrivate);
+	self->priv = gth_image_viewer_page_get_instance_private (self);
 	self->priv->settings = g_settings_new (GTHUMB_IMAGE_VIEWER_SCHEMA);
 	self->priv->preloader = NULL;
 	self->priv->file_popup_merge_id = 0;
@@ -2161,7 +2194,7 @@ gth_image_viewer_page_paste_image (GthImageViewerPage *self)
 
 typedef struct {
 	GthImageViewerPage *viewer_page;
-	GSimpleAsyncResult *result;
+	GTask              *task;
 	GCancellable       *cancellable;
 } OriginalImageData;
 
@@ -2172,8 +2205,8 @@ get_original_data_new (void)
 	OriginalImageData *data;
 
 	data = g_new0 (OriginalImageData, 1);
-	data->result = NULL;
 	data->cancellable = NULL;
+	data->task = NULL;
 
 	return data;
 }
@@ -2187,7 +2220,7 @@ get_original_data_free (OriginalImageData *data)
 
 	_g_object_unref (data->viewer_page);
 	_g_object_unref (data->cancellable);
-	_g_object_unref (data->result);
+	_g_object_unref (data->task);
 	g_free (data);
 }
 
@@ -2202,8 +2235,7 @@ original_image_ready_cb (GObject	*source_object,
 	GError            *error = NULL;
 
 	if (! _gth_image_viewer_page_load_with_preloader_finish (data->viewer_page)) {
-		g_simple_async_result_take_error (data->result, g_error_new_literal (G_IO_ERROR, G_IO_ERROR_CANCELLED, ""));
-		g_simple_async_result_complete_in_idle (data->result);
+		g_task_return_error (data->task, g_error_new_literal (G_IO_ERROR, G_IO_ERROR_CANCELLED, ""));
 		get_original_data_free (data);
 		return;
 	}
@@ -2217,13 +2249,10 @@ original_image_ready_cb (GObject	*source_object,
 					       NULL,
 					       &error))
 	{
-		g_simple_async_result_take_error (data->result, error);
+		g_task_return_error (data->task, error);
 	}
 	else
-		g_simple_async_result_set_op_res_gpointer (data->result,
-							   image,
-							   (GDestroyNotify) g_object_unref);
-	g_simple_async_result_complete_in_idle (data->result);
+		g_task_return_pointer (data->task, image, (GDestroyNotify) g_object_unref);
 
 	get_original_data_free (data);
 }
@@ -2239,20 +2268,17 @@ gth_image_viewer_page_get_original (GthImageViewerPage	 *self,
 
 	data = get_original_data_new ();
 	data->viewer_page = g_object_ref (self);
-	data->result = g_simple_async_result_new (G_OBJECT (self),
-						  ready_callback,
-						  user_data,
-						  gth_image_viewer_page_get_original);
 	data->cancellable = (cancellable != NULL) ? g_object_ref (cancellable) : g_cancellable_new ();
+	data->task = g_task_new (G_OBJECT (self),
+				 data->cancellable,
+				 ready_callback,
+				 user_data);
 
 	if (gth_image_viewer_is_animation (GTH_IMAGE_VIEWER (self->priv->viewer))) {
 		GthImage *image;
 
 		image = gth_image_new_for_surface (gth_image_viewer_get_current_image (GTH_IMAGE_VIEWER (self->priv->viewer)));
-		g_simple_async_result_set_op_res_gpointer (data->result,
-							   image,
-							   (GDestroyNotify) g_object_unref);
-		g_simple_async_result_complete_in_idle (data->result);
+		g_task_return_pointer (data->task, image, (GDestroyNotify) g_object_unref);
 
 		get_original_data_free (data);
 	}
@@ -2275,17 +2301,17 @@ gth_image_viewer_page_get_original_finish (GthImageViewerPage	 *self,
 {
 	GthImage *image;
 
-	g_return_val_if_fail (g_simple_async_result_is_valid (result, G_OBJECT (self), gth_image_viewer_page_get_original), FALSE);
+	g_return_val_if_fail (g_task_is_valid (G_TASK (result), G_OBJECT (self)), FALSE);
 
-	if (g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (result), error))
+	image = g_task_propagate_pointer (G_TASK (result), error);
+	if (image == NULL)
 		return FALSE;
 
-	image = g_simple_async_result_get_op_res_gpointer (G_SIMPLE_ASYNC_RESULT (result));
-	g_return_val_if_fail (image != NULL, FALSE);
-
 	if (image_p != NULL)
 		*image_p = gth_image_get_cairo_surface (image);
 
+	g_object_unref (image);
+
 	return TRUE;
 }
 
