@@ -43,12 +43,6 @@
 #define BLACK_VALUE 0.2
 
 
-G_DEFINE_TYPE_WITH_CODE (GthImageViewer,
-			 gth_image_viewer,
-			 GTK_TYPE_WIDGET,
-                         G_IMPLEMENT_INTERFACE (GTK_TYPE_SCROLLABLE, NULL))
-
-
 enum {
 	PROP_0,
 	PROP_HADJUSTMENT,
@@ -125,6 +119,13 @@ struct _GthImageViewerPrivate {
 static guint gth_image_viewer_signals[LAST_SIGNAL] = { 0 };
 
 
+G_DEFINE_TYPE_WITH_CODE (GthImageViewer,
+			 gth_image_viewer,
+			 GTK_TYPE_WIDGET,
+			 G_ADD_PRIVATE (GthImageViewer)
+			 G_IMPLEMENT_INTERFACE (GTK_TYPE_SCROLLABLE, NULL))
+
+
 typedef struct {
 	GthImageViewerPaintFunc func;
 	gpointer                user_data;
@@ -442,7 +443,7 @@ gth_image_viewer_realize (GtkWidget *widget)
 				  | GDK_BUTTON_MOTION_MASK
 				  | GDK_SCROLL_MASK
 				  | GDK_STRUCTURE_MASK);
-	attributes_mask = GDK_WA_X | GDK_WA_Y | GDK_WA_VISUAL;
+	attributes_mask = GDK_WA_X | GDK_WA_Y | GDK_WA_VISUAL | GDK_WA_WMCLASS;
 	window = gdk_window_new (gtk_widget_get_parent_window (widget),
 				 &attributes,
 				 attributes_mask);
@@ -450,8 +451,8 @@ gth_image_viewer_realize (GtkWidget *widget)
 	gtk_widget_set_window (widget, window);
 	gtk_style_context_set_background (gtk_widget_get_style_context (widget), window);
 
-	self->priv->cursor = gdk_cursor_new (GDK_LEFT_PTR);
-	self->priv->cursor_void = gdk_cursor_new_for_display (gtk_widget_get_display (widget), GDK_BLANK_CURSOR);
+	self->priv->cursor = _gdk_cursor_new_for_widget (widget, GDK_LEFT_PTR);
+	self->priv->cursor_void = _gdk_cursor_new_for_widget (widget, GDK_BLANK_CURSOR);
 	if (self->priv->cursor_visible)
 		gdk_window_set_cursor (window, self->priv->cursor);
 	else
@@ -1239,8 +1240,6 @@ gth_image_viewer_class_init (GthImageViewerClass *class)
 	GtkWidgetClass *widget_class;
 	GtkBindingSet  *binding_set;
 
-	g_type_class_add_private (class, sizeof (GthImageViewerPrivate));
-
 	/* signals */
 
 	gth_image_viewer_signals[CLICKED] =
@@ -1513,12 +1512,11 @@ static void
 gth_image_viewer_init (GthImageViewer *self)
 {
 	gtk_widget_set_can_focus (GTK_WIDGET (self), TRUE);
-	gtk_widget_set_double_buffered (GTK_WIDGET (self), TRUE);
 	gtk_widget_set_has_window (GTK_WIDGET (self), TRUE);
 
 	/* Initialize data. */
 
-	self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, GTH_TYPE_IMAGE_VIEWER, GthImageViewerPrivate);
+	self->priv = gth_image_viewer_get_instance_private (self);
 
 	self->priv->is_animation = FALSE;
 	self->priv->play_animation = TRUE;
