@@ -36,9 +36,6 @@
 #define MIN_WIDTH 200
 
 
-G_DEFINE_TYPE (GthLocationChooser, gth_location_chooser, GTK_TYPE_BOX)
-
-
 enum {
 	ITEM_TYPE_NONE,
 	ITEM_TYPE_SEPARATOR,
@@ -55,11 +52,10 @@ enum {
 	N_COLUMNS
 };
 
-
 enum {
-        PROP_0,
-        PROP_SHOW_ENTRY_POINTS,
-        PROP_RELIEF
+	PROP_0,
+	PROP_SHOW_ENTRY_POINTS,
+	PROP_RELIEF
 };
 
 enum {
@@ -67,10 +63,9 @@ enum {
 	LAST_SIGNAL
 };
 
-struct _GthLocationChooserPrivate
-{
+
+struct _GthLocationChooserPrivate {
 	GtkWidget      *combo;
-	GtkWidget      *arrow;
 	GtkTreeStore   *model;
 	GFile          *location;
 	GthFileSource  *file_source;
@@ -86,6 +81,11 @@ struct _GthLocationChooserPrivate
 static guint gth_location_chooser_signals[LAST_SIGNAL] = { 0 };
 
 
+G_DEFINE_TYPE_WITH_CODE (GthLocationChooser,
+			 gth_location_chooser,
+			 GTK_TYPE_BOX,
+			 G_ADD_PRIVATE (GthLocationChooser))
+
 
 static void
 gth_location_chooser_set_property (GObject      *object,
@@ -492,8 +492,6 @@ gth_location_chooser_class_init (GthLocationChooserClass *klass)
 	GObjectClass   *object_class;
 	GtkWidgetClass *widget_class;
 
-	g_type_class_add_private (klass, sizeof (GthLocationChooserPrivate));
-
 	object_class = (GObjectClass*) klass;
 	object_class->set_property = gth_location_chooser_set_property;
 	object_class->get_property = gth_location_chooser_get_property;
@@ -535,16 +533,6 @@ gth_location_chooser_class_init (GthLocationChooserClass *klass)
 }
 
 
-static void
-get_combo_box_button (GtkWidget *widget,
-		      gpointer   data)
-{
-	GtkWidget **p_child = data;
-
-	if (GTK_IS_BUTTON (widget))
-		*p_child = widget;
-}
-
 static void
 gth_location_chooser_init (GthLocationChooser *self)
 {
@@ -553,9 +541,8 @@ gth_location_chooser_init (GthLocationChooser *self)
 	gtk_widget_set_can_focus (GTK_WIDGET (self), FALSE);
 	gtk_orientable_set_orientation (GTK_ORIENTABLE (self), GTK_ORIENTATION_HORIZONTAL);
 
-	self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, GTH_TYPE_LOCATION_CHOOSER, GthLocationChooserPrivate);
+	self->priv = gth_location_chooser_get_instance_private (self);
 	self->priv->entry_points_changed_id = 0;
-	self->priv->arrow = NULL;
 	self->priv->show_entry_points = TRUE;
 	self->priv->relief = GTK_RELIEF_NORMAL;
 	self->priv->reload = FALSE;
@@ -621,71 +608,14 @@ gth_location_chooser_new (void)
 /* -- gth_location_chooser_set_relief -- */
 
 
-static void
-get_combo_box_arrow (GtkWidget *widget,
-		      gpointer   data)
-{
-	GtkWidget **p_child = data;
-
-	if (GTK_IS_ARROW (widget))
-		*p_child = widget;
-}
-
-
-static gboolean
-show_combo_box_arrow (GthLocationChooser *self)
-{
-	if ((self->priv->arrow != NULL) && (self->priv->relief == GTK_RELIEF_NONE))
-		gtk_widget_show (self->priv->arrow);
-
-	return FALSE;
-}
-
-
-static gboolean
-hide_combo_box_arrow (GthLocationChooser *self)
-{
-	if ((self->priv->arrow != NULL) && (self->priv->relief == GTK_RELIEF_NONE))
-		gtk_widget_hide (self->priv->arrow);
-
-	return FALSE;
-}
-
-
 void
 gth_location_chooser_set_relief (GthLocationChooser *self,
 				 GtkReliefStyle      value)
 {
-	GtkWidget *button;
-
 	if (self->priv->relief == value)
 		return;
 
 	self->priv->relief = value;
-
-	button = NULL;
-	gtk_container_forall (GTK_CONTAINER (self->priv->combo), get_combo_box_button, &button);
-	if (button != NULL) {
-		gtk_button_set_relief (GTK_BUTTON (button), self->priv->relief);
-
-		/* show the arrow only when the pointer is over the combo_box */
-
-		if (self->priv->arrow == NULL) {
-			gtk_container_forall (GTK_CONTAINER (gtk_bin_get_child (GTK_BIN (button))), get_combo_box_arrow, &self->priv->arrow);
-			g_signal_connect_swapped (button,
-					  	  "enter-notify-event",
-					  	  G_CALLBACK (show_combo_box_arrow),
-					  	  self);
-			g_signal_connect_swapped (button,
-					  	  "leave-notify-event",
-					  	  G_CALLBACK (hide_combo_box_arrow),
-					  	  self);
-		}
-
-		if (self->priv->arrow != NULL)
-			gtk_widget_set_visible (self->priv->arrow, self->priv->relief != GTK_RELIEF_NONE);
-	}
-
 	g_object_notify (G_OBJECT (self), "relief");
 }
 
